@@ -4,6 +4,7 @@ import asyncio
 import hashlib
 import json
 import logging
+import os
 import ssl
 import subprocess
 import time
@@ -12,6 +13,9 @@ from enum import Enum
 from typing import Optional, Tuple
 from pathlib import Path
 
+# Logger for this module
+logger = logging.getLogger(__name__)
+
 import websockets
 from websockets.server import WebSocketServerProtocol
 from websockets.http import Headers
@@ -134,11 +138,80 @@ class Session:
     permission_level: str = "full"  # "full" or "read_only"
     # Chat history
     chat_session_id: str = ""  # Persistent chat session ID for resuming
+    # Claude session ID for --resume functionality
+    claude_session_id: str = ""  # Claude Code session to resume
 
     def __post_init__(self):
         self.working_dir = Path(self.working_dir)
 
 
+def list_claude_sessions(working_dir: Path) -> list[dict]:
+    """List available Claude Code sessions for a working directory.
+
+    Scans ~/.claude/projects/ for session files matching the working directory.
+    Returns sessions sorted by last modified time (most recent first).
+    """
+    claude_dir = Path.home() / ".claude" / "projects"
+    if not claude_dir.exists():
+        return []
+
+    # Convert working dir to Claude's naming convention (path with dashes)
+    # e.g., /Users/foo/bar -> -Users-foo-bar
+    safe_name = str(working_dir.resolve()).replace("/", "-")
+    project_dir = claude_dir / safe_name
+
+    if not project_dir.exists():
+        return []
+
+    sessions = []
+    for session_file in project_dir.glob("*.jsonl"):
+        # Skip agent sessions (subagent files)
+        if session_file.stem.startswith("agent-"):
+            continue
+
+        try:
+            stat = session_file.stat()
+            # Try to read first and last line for metadata
+            preview = ""
+            session_id = session_file.stem
+
+            # Check if there's a subdirectory with same name (active session)
+            has_subdir = (project_dir / session_id).is_dir()
+
+            with open(session_file, "r") as f:
+                first_line = f.readline()
+                if first_line:
+                    try:
+                        data = json.loads(first_line)
+                        if data.get("type") == "user":
+                            msg = data.get("message", {})
+                            content = msg.get("content", [])
+                            if isinstance(content, list) and content:
+                                for block in content:
+                                    if isinstance(block, dict) and block.get("type") == "text":
+                                        text = block.get("text", "")
+                                        # Skip system messages
+                                        if not text.startswith("<"):
+                                            preview = text[:100]
+                                            break
+                    except json.JSONDecodeError:
+                        pass
+
+            sessions.append({
+                "sessionId": session_id,
+                "lastModified": int(stat.st_mtime * 1000),
+                "size": stat.st_size,
+                "preview": preview,
+                "isActive": has_subdir,
+            })
+        except (IOError, OSError):
+            continue
+
+    # Sort by last modified (most recent first)
+    sessions.sort(key=lambda x: x["lastModified"], reverse=True)
+    return sessions[:20]  # Limit to 20 most recent
+
+
 class DarkCodeServer:
     """WebSocket server that bridges mobile app to Claude Code CLI."""
 
@@ -316,9 +389,6 @@ class DarkCodeServer:
 
         # Handle admin dashboard requests (non-WebSocket HTTP)
         if isinstance(path, str) and path.startswith('/admin'):
-            # Debug: print the full path to terminal
-            print(f"[DEBUG] Admin request path: {path!r}")
-
             # Initialize web admin handler if needed
             if self._web_admin is None:
                 try:
@@ -327,10 +397,8 @@ class DarkCodeServer:
                 except ImportError:
                     return Response(404, "Not Found", Headers([]), b"Web admin not available")
 
-            # Determine HTTP method and gather request info
-            method = 'GET'  # Default
-            # Note: POST detection is limited in websockets process_request
-            # We'll use a workaround with content-length header
+            # Determine HTTP method
+            method = 'GET'
             content_length = request_headers.get('Content-Length', '0')
             if int(content_length) > 0:
                 method = 'POST'
@@ -338,31 +406,20 @@ class DarkCodeServer:
             # Build headers dict
             headers = {str(k): str(v) for k, v in request_headers.items()}
 
-            # For POST requests, we can't easily get body in process_request
-            # So we handle it via query params or redirect to GET
-            body = b''
-
             # Handle the request
-            print(f"[DEBUG] Calling handle_request with path={path!r}, method={method}")
             status, resp_headers, resp_body = self._web_admin.handle_request(
-                path, method, headers, body
+                path, method, headers, b''
             )
-            print(f"[DEBUG] Response: status={status}, headers={resp_headers}")
 
             # Build Response object for websockets 13+
-            # Get status phrase
             from http import HTTPStatus
             try:
                 reason = HTTPStatus(status).phrase
             except ValueError:
                 reason = "Unknown"
 
-            # Convert headers to Headers object
             header_list = [(k, v) for k, v in resp_headers.items()]
-            print(f"[DEBUG] Header list: {header_list}")
-            response_headers = Headers(header_list)
-
-            return Response(status, reason, response_headers, resp_body)
+            return Response(status, reason, Headers(header_list), resp_body)
 
         # Check if this is a WebSocket upgrade request
         # If not, return a friendly HTTP response instead of letting websockets raise InvalidUpgrade
@@ -693,11 +750,15 @@ class DarkCodeServer:
                 else:
                     self.ip_session_count[client_ip] = count
 
-    async def _start_claude_process(self, session: Session):
+    async def _start_claude_process(self, session: Session, resume_session_id: str = None):
         """Start the Claude Code CLI process.
 
         SECURITY: Uses validated working directory from config.
         Uses -p (print mode) with stream-json for bidirectional streaming.
+
+        Args:
+            session: The session to start Claude for
+            resume_session_id: Optional Claude session ID to resume (for handoffs)
         """
         try:
             # Use validated working directory
@@ -714,6 +775,10 @@ class DarkCodeServer:
                 "--permission-mode", self.config.permission_mode,
             ]
 
+            # Add resume flag if resuming a session (session handoff)
+            if resume_session_id:
+                cmd.extend(["--resume", resume_session_id])
+
             session.process = subprocess.Popen(
                 cmd,
                 cwd=str(working_dir),
@@ -1216,6 +1281,45 @@ class DarkCodeServer:
                     "chatSessionId": chat_session_id,
                 }))
 
+        # === Claude Session Handoffs ===
+        elif msg_type == "list_claude_sessions":
+            # List available Claude Code sessions for handoff
+            sessions_list = list_claude_sessions(session.working_dir)
+            await session.websocket.send(json.dumps({
+                "type": "claude_sessions_list",
+                "sessions": sessions_list,
+                "workingDir": str(session.working_dir),
+            }))
+
+        elif msg_type == "resume_claude_session":
+            # Resume a specific Claude Code session
+            claude_session_id = msg.get("sessionId", "")
+            if not claude_session_id:
+                await session.websocket.send(json.dumps({
+                    "type": "error",
+                    "message": "sessionId is required to resume a session",
+                }))
+                return
+
+            # Kill existing Claude process if running
+            if session.process and self._is_process_alive(session):
+                try:
+                    session.process.terminate()
+                    session.process.wait(timeout=3)
+                except Exception:
+                    session.process.kill()
+
+            # Store the Claude session ID for resume
+            session.claude_session_id = claude_session_id
+
+            # Start Claude with the resume flag
+            await self._start_claude_process(session, resume_session_id=claude_session_id)
+
+            await session.websocket.send(json.dumps({
+                "type": "claude_session_resumed",
+                "sessionId": claude_session_id,
+            }))
+
         # === File Operations ===
         elif msg_type == "list_files":
             # List files in a directory
@@ -1241,6 +1345,7 @@ class DarkCodeServer:
         elif msg_type == "execute_bash":
             command = msg.get("command", "")
             timeout = msg.get("timeout", 30000)  # Default 30s
+            logger.info(f"[BASH] Received execute_bash: {command[:100]}")
             await self._handle_execute_bash(session, command, timeout)
 
     async def _handle_execute_bash(self, session: Session, command: str, timeout_ms: int):
@@ -1303,14 +1408,16 @@ class DarkCodeServer:
                         output += "\n"
                     output += stderr.decode("utf-8", errors="replace")
 
-                await session.websocket.send(json.dumps({
+                response = {
                     "type": "bash_output",
                     "command": command,
                     "output": output.strip() or "(no output)",
                     "exitCode": exit_code,
                     "isError": exit_code != 0,
                     "timestamp": int(time.time() * 1000),
-                }))
+                }
+                logger.info(f"[BASH] Sending response: exit={exit_code}, output_len={len(output)}")
+                await session.websocket.send(json.dumps(response))
 
             except asyncio.TimeoutError:
                 process.kill()
